# -*- coding: UTF-8 -*-
from py.test import raises
import exceptions
from scribe import backend
from os.path import isdir, isfile, islink
from os import system, listdir
from creole import parsedico
from creole.eosfunc import test_vz
from time import sleep
import ftplib, imaplib

dico = parsedico.parse_dico()
dyn_logon = """python /usr/share/eole/backend/dyn_logon.py --user="%s" --os="%s" --machine="%s" --ip="%s" """

def test_create_niveau():
    backend.ajout_groupe('tnivo', 'niveau', domaine='', optrw='')

def test_create_classe():
    backend.ajout_classe('tclasse', 'tnivo')
    assert isdir('/home/workgroups/tclasse/donnees')
    assert isdir('/home/workgroups/profs-tclasse')
    assert 'tnivo' == backend.get_niveau('tclasse')

def test_create_gropupe():
    backend.ajout_groupe('tgroupe', 'groupe', domaine='', optrw='dt')
    assert isdir('/home/workgroups/tgroupe/travail')

def test_create_eleve():
    backend.ajout_eleve('televe', 'tpass', 'nom', 'prenom', '01021980', 'tclasse', '666',
                  civilite='1', domaine='restreint', quota='20', profil='3', shell=False)
    assert isdir('/home/t/televe/MailDir')
    # liens classe -> eleves
    assert isdir('/home/workgroups/profs-tclasse/eleves/televe')

def test_create_prof():
    backend.ajout_prof('tprof', 'tpass', 'nom', 'prenom', '01021980', classe='tclasse', civilite='1',
               mail='local', quota='0', profil='1', shell=False)
    assert isdir('/home/t/tprof/MailDir')

def test_inscription_prof():
    backend.inscription('tprof', 'profs-tclasse')
    assert isdir('/home/t/tprof/.ftp/profs-tclasse')

def test_inscription_eleve():
    backend.inscription('televe', 'tgroupe')
    assert isdir('/home/t/televe/.ftp/tgroupe')

def test_smb_perso_eleve():
    if not test_vz():
        system('mkdir -p /tmp/home')
        assert system('smbmount //127.0.0.1/televe /tmp/home -o username=televe,password=tpass') == 0
        system('touch /tmp/home/fichier.txt')
        assert isfile('/tmp/home/fichier.txt')
        system('rm -f /tmp/home/fichier.txt')
        assert not isfile('/tmp/home/fichier.txt')
        # test antivirus temps réel
        if dico['smb_vscan'] == 'oui':
            system("""echo 'X5O!P%@AP[4\PZX54(P^)7CC)7}$EICAR-STANDARD-ANTIVIRUS-TEST-FILE!$H+H*' > /tmp/home/virus.txt""")
            sleep(1)
            assert not isfile('/tmp/home/virus.txt')
            system('rm -f /tmp/home/virus.txt')
            # -> plante rsyslog ;)
            #system('sed -i "/\/home\/t\/televe\/virus.txt/d" /var/log/syslog')
        system('umount /tmp/home/')

def test_smb_classe_eleve():
    if not test_vz():
        system('mkdir -p /tmp/classe')
        assert system('smbmount //127.0.0.1/tclasse /tmp/classe -o username=televe,password=tpass') == 0
        system('touch /tmp/classe/travail/fichier.txt')
        assert isfile('/tmp/classe/travail/fichier.txt')
        system('rm -f /tmp/classe/travail/fichier.txt')
        assert not isfile('/tmp/classe/travail/fichier.txt')
        system('touch /tmp/classe/donnees/fichier.txt')
        assert not isfile('/tmp/classe/donnees/fichier.txt')
        system('rm -f /tmp/classe/donnees/fichier.txt')
        system('umount /tmp/classe')

def test_smb_equipe_prof():
    if not test_vz():
        system('mkdir -p /tmp/equipe')
        assert system('smbmount //127.0.0.1/profs-tclasse /tmp/equipe -o username=tprof,password=tpass') == 0
        assert isdir('/tmp/equipe/eleves/televe')
        if dico['smb_unixextensions'] != 'yes':
            # pas le droit de scanner le privé (FIXME : problème avec unixextensions)
            raises(exceptions.OSError, listdir, '/tmp/equipe/eleves/televe/prive')
        system('umount /tmp/equipe')

def test_logon95():
    system(dyn_logon % ('tprof', 'Win95', 'test95', '1.1.1.1'))
    assert isfile('/home/netlogon/tprofWin95.bat')
    assert "NET USE  U: \\\\%s\PERSO" % dico['smb_netbios_name'] in \
           file('/home/netlogon/tprofWin95.bat').read().splitlines()
    system('rm -f /home/netlogon/tprofWin95.bat')

def test_logonXP():
    system(dyn_logon % ('tprof', 'WinXP', 'testXP', '1.1.1.2'))
    assert isfile('/home/netlogon/tprofWinXP.txt')
    assert "lecteur,P:,\\\\%s\professeurs" % dico['smb_netbios_name'] in \
           file('/home/netlogon/tprofWinXP.txt').read().splitlines()
    system('rm -f /home/netlogon/tprofWinXP.txt')

def test_ftp_eleve():
    if dico['ftp_perso'] == 'oui':
        session = ftplib.FTP('127.0.0.1', 'televe', 'tpass')
        session.quit()

def test_mail_eleve():
    mail = imaplib.IMAP4()
    mail.login('televe', 'tpass')
    mail.select('inbox')
    assert mail.fetch(1, "(body[header])")[1][0][1].startswith('Received:')
    mail.close()
    mail.logout()

def test_mail_prof():
    mail = imaplib.IMAP4()
    mail.login('tprof', 'tpass')
    mail.select('inbox')
    assert mail.fetch(1, "(body[header])")[1][0][1].startswith('Received:')
    mail.close()
    mail.logout()

def test_mod_mail():
    backend.mod_mail('tprof', mailtype='perso', mail='tprof@tprof.com')
    assert backend.get_attr('tprof', 'mail') == ['tprof@tprof.com']
    assert backend.get_attr('tprof', 'mailDir') == []
    assert backend.get_attr('tprof', 'mailHost') == []

def test_profils():
    # profils à la création
    assert backend.get_attr('televe', 'sambaProfilePath') == ["\\\\%(smb_netbios_name)s\\netlogon\\profil2" % dico]
    assert backend.get_attr('tprof', 'sambaProfilePath') == []
    # modification de profils
    backend.set_profil('tprof', 2)
    assert backend.get_attr('tprof', 'sambaProfilePath') == ["\\\\%(smb_netbios_name)s\\netlogon\\profil" % dico]
    backend.set_profil('tprof', 4)
    assert backend.get_attr('tprof', 'sambaProfilePath') == ["\\\\%(smb_netbios_name)s\\tprof\\profil" % dico]
    backend.set_profil('tprof', 1)
    assert backend.get_attr('tprof', 'sambaProfilePath') == []

def test_quotas():
    # quotas à la création
    assert backend.get_quota('televe') == 20
    assert backend.get_quota('tprof') == 0
    # modification de quotas
    backend.set_quota('tprof', 50)
    assert backend.get_quota('tprof') == 50

def test_desinscription():
    backend.desinscription('televe', 'tgroupe')
    assert not isdir('/home/t/televe/.ftp/tgroupe')

def test_delete_eleve():
    backend.del_user('televe', remove_data=True)
    assert not isdir('/home/t/televe')
    assert not islink('/home/tclasse/televe')

def test_delete_prof():
    backend.del_user('tprof', remove_data=True)
    assert not isdir('/home/t/tprof')

def test_delete_shares():
    backend.del_share('tclasse', rmdir=True, sync=False)
    assert not isdir('/home/workgroups/tclasse')
    backend.del_share('profs-tclasse', rmdir=True, sync=False)
    assert not isdir('/home/workgroups/profs-tclasse')
    backend.del_share('tgroupe', rmdir=True, sync=True)
    assert not isdir('/home/workgroups/tgroupe')

def test_delete_classe():
    #system("rm -rf /var/lib/sympa/expl/i-*/tclasse/")
    backend.del_group('tclasse')
    assert not isdir('/home/workgroups/profs-tclasse')
    assert not isdir('/home/classes/tclasse')

def test_delete_groups():
    backend.del_group('tnivo')
    backend.del_group('tgroupe')

