# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
###########################################################################
"""
    interface de gestion des user 'administratifs'
"""
from os import system, makedirs
from os.path import join, isdir
from ldap import MOD_REPLACE
from scribe.ldapconf import USER_DN, SMB_SERVEUR, \
HOME_PATH, MAIL_DOMAIN, ADMINISTRATIF_FILTER
from scribe.eoletools import set_default_acl, add_dir,  \
format_current_date, not_empty, to_list
from scribe.eoleuser import User, gen_common_attrs, send_first_mail, \
gen_profil, gen_radius_attrs

class Administratif(User):
    """
        classe pour la gestion des personnels administratifs
    """
    _type = 'personnels'
    profil = 'administratif'
    filtre = "(&%s)" % ADMINISTRATIF_FILTER
    must_args = ['login', 'password', 'nom', 'prenom', 'date', 'civilite']
    may_args = dict(mail='internet',
                    quota='0',
                    profil='1',
                    shell=False,
                    change_pwd=False,
                    mail_acad='',
                    int_id='',
                    groupe='',
                    entlogin=True,
                    )
    has_samba = True
    has_ftp = True
    user_add_params = """-a -A 1 -D "U:" -H U -o "ou=local,ou=personnels" \
-s /bin/%%(shell)s \
-d "/home/%%(login_first)s/%%(login)s" \
-C '\\\\%s\\%%(login)s\\perso' \
-F '%%(profil)s' -g "administratifs" \
-G "%%(groups)s" \
"%%(login)s" """ % SMB_SERVEUR

    def get_smbldap_useradd_args(self, login,  profil, shell, groupe, **args):
        """
            ajoute un utilisateur à l'aide /usr/sbin/smbldap-useradd
        """
        if shell:
            shell = "bash"
        else:
            shell = "false"
        groups = to_list(groupe)
        groups.append('DomainUsers')
        arg_dict = {"shell":shell,
                    "login_first":login[0],
                    "login":login,
                    "profil":gen_profil(profil, login),
                    "groups":','.join(groups),
                    }
        return arg_dict


    def _add_scribe_user(self, login, **args):
        """
            ajout les attributs Scribe à un Administratif
        """
        mail = args['mail']
        if '@' not in mail:
            if mail == 'local':
                # compatibilité avec l'ancienne valeur
                mail = 'internet'
            if mail in ['internet', 'restreint']:
                mail = "%s@%s" % (login, MAIL_DOMAIN[mail])
                maildir = self.get_maildir(login)
            else:
                mail = ''
                maildir = ''
        else:
            maildir = ''
        # Rappel : 0-> prof normal, 1-> utilisateur admin,
        #          2-> prof principal, 3-> administratifs
        typeadmin = '3'
        datas = []
        user_dn = USER_DN % dict(uid=login, _type=self._type)
        objectclass = self._get_attr(login, 'objectClass')
        objectclass.extend(['administratif', 'ENTPerson',
            'ENTAuxNonEnsEtab', 'radiusprofile'])
        datas.append((MOD_REPLACE, 'objectClass', objectclass))
        datas.extend(gen_common_attrs(login, entprofil=self.profil, **args))
        datas.extend(gen_radius_attrs())
        datas.append((MOD_REPLACE, 'mail', mail))
        if maildir != '':
            datas.append((MOD_REPLACE, 'mailHost', 'localhost'))
            datas.append((MOD_REPLACE, 'mailDir', maildir))
        datas.append((MOD_REPLACE, 'typeadmin', typeadmin))
        if not_empty(args, 'mail_acad'):
            datas.append((MOD_REPLACE, 'FederationKey',
                          args['mail_acad'].lower()))
        # FIXME : remplissage d'attributs provenant de ENTAuxNonEnsEtab
        self.ldap_admin._modify(user_dn, datas)
        if maildir != '':
            send_first_mail(mail)

    def _add_perso(self, login, **args):
        """
            Crée les différents répertoires de l'utilisateur
        """
        # remplace /usr/share/ead/rep-prof.sh
        abc = join(HOME_PATH, login[0])
        rep = join(abc, login)
        perso = join(rep, 'perso')
        maildir = join(rep, 'MailDir')
        if not isdir(abc):
            makedirs(abc)
        system("setfacl -m u:mail:r-x,g:mail:r-x %s" % (abc,))
        add_dir(perso)
        add_dir(maildir)
        system("setfacl -PRm u:%s:rwx,g::---,o::--- %s" % (login, rep))
        # mail
        system("setfacl -m u:mail:r-x,g:mail:r-x %s" % (rep,))
        system("setfacl -m u:mail:rwx,g:mail:rwx %s" % (maildir,))
        # acl par défaut
        set_default_acl(rep, recursive=False)
        set_default_acl(perso, recursive=False)
        set_default_acl(maildir, recursive=False)
        # pour les quotas
        system('chown -R %s %s' % (login, perso))
        # pour courier-imap (hardy)
        system('chown mail:mail %s' % (maildir,))

    def _update(self, login, **args):
        """
            Mise à niveau Administratif via l'extraction
        """
        user_dn = USER_DN % dict(uid=login, _type=self._type)
        datas = []
        if not_empty(args, 'mail_acad'):
            datas.append((MOD_REPLACE, 'FederationKey',
                          args['mail_acad'].lower()))
        if not_empty(args, 'int_id'):
            datas.append((MOD_REPLACE, 'intid', args['int_id'] ))
        datas.append((MOD_REPLACE, 'LastUpdate', format_current_date()))
        self.ldap_admin._modify(user_dn, datas)

    def _get_administratifs(self, attrs=['uid']):
        return self._get_users(filtre=self.filtre, attrs=attrs)

