#-*-coding:utf-8-*-
###########################################################################
#
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
#
###########################################################################
"""
Outils de mise à niveau de l'annuaire Scribe
"""
import sys
from os import unlink
from os.path import isfile, join
from time import sleep
from ldap import ALREADY_EXISTS, NO_SUCH_OBJECT
from creole.utils import print_red
from scribe.eoleldap import Ldap
from scribe.ldapconf import BRANCHE, GROUP_FILTER, ELEVE_FILTER, PROF_FILTER,\
GROUP_DN, SMB_SERVEUR
from scribe.errors import LdapExistingGroup
from scribe.eoleshare import Share
from scribe.eolegroup import Group
from scribe.eleves import Eleve
from scribe.autres import Autre
from scribe.enseignants import Enseignant

def add_branches():
    """
        Ajout des branches (o.u.) manquantes
    """
    connexion = Ldap()
    connexion.connect()

    for unit in ['autres', 'responsables']:
        dn = "ou=%s,ou=utilisateurs,%s" % (unit, BRANCHE)
        data = [("objectClass","organizationalUnit"),
                ("ou", unit),]
        try:
            connexion._add(dn, data)
            print "ajout de %s" % dn
        except ALREADY_EXISTS:
            pass
        dn = "ou=local,ou=%s,ou=utilisateurs,%s" % (unit, BRANCHE)
        data = [("objectClass","organizationalUnit"),
                ("ou", "local"),]
        try:
            connexion._add(dn, data)
            print "ajout de %s" % dn
        except ALREADY_EXISTS:
            pass

def upgrade_special_shares():
    """
        transfert des lettres de lecteurs
        du fichier special_shares.py
        vers l'attribut ldap sambaShareDrive
    """
    special_path = '/usr/share/eole/backend/'
    special_file = join(special_path, 'special_shares.py')
    if isfile(special_file):
        sys.path.append('/usr/share/eole/backend/')
        import special_shares as shares
        share = Share()
        share.ldap_admin.connect()
        # partages EOLE
        for sharename, drive in shares.eole_shares.items():
            #print "%s -> %s" % (sharename, drive)
            # utilisation de _set_attr au lieu de _set_share_drive
            # pour contourner les exceptions ;)
            try:
                share._set_attr(sharename, 'sambaShareDrive', drive)
            except NO_SUCH_OBJECT:
                print_red ("ERREUR : partage spécial smb://%s/%s non trouvé dans l'annuaire" % (SMB_SERVEUR, sharename))
                sys.exit(1)
        # partages utilisateur
        for sharename, drive in shares.special_shares.items():
            #print "%s -> %s" % (sharename, drive)
            try:
                share._set_attr(sharename, 'sambaShareDrive', drive)
            except NO_SUCH_OBJECT:
                print "info : partage %s non présent dans l'annuaire" % sharename
        unlink(special_file)

def add_missing_groups():
    """
        ajout des groupes manquants
    """
    grp = Group()
    grp.ldap_admin.connect()
    # ajout du groupe "administratifs"
    # (version basic avec gid=10000)
    name = 'administratifs'
    try:
        grp._test_available_name(name)
    except LdapExistingGroup:
        return
    grp_dn = GROUP_DN % {'cn':name}
    sid = grp._get_attr('eleves', 'sambaSID')[0].split('-')
    sid[-1] = "21001"
    sid = '-'.join(sid)
    datas = [('objectClass', ['posixGroup', 'sambaGroupMapping']),
    ('cn', name),
    ('gidNumber', '10000'),
    ('description', 'Personnel administratif'),
    ('sambaSID', sid),
    ('sambaGroupType', '2'),
    ]
    grp.ldap_admin._add(grp_dn, datas)
    grp._add_maillist('Groupe', name, 'restreint')
    grp.ldap_admin.close()

def upgrade_groups():
    """
        mise à niveau des groupes
    """
    grp = Group()
    grp.ldap_admin.connect()
    filtre = "(&%s(!(objectClass=eolegroupe)))" % GROUP_FILTER
    groups = grp.ldap_admin._search(filtre, ['cn'])
    for group in groups:
        name = group[1]['cn'][0]
        if name in ['DomainAdmins', 'DomainUsers', 'DomainComputers', 'PrintOperators']:
            # on ne touche pas aux groupes Windows
            continue
        try:
            grp._Upgrade(name)
        except Exception, message:
            print "Mise à niveau du groupe %s impossible :" % name
            print str(message) + '\n'
    grp.ldap_admin.close()

def upgrade_eleves():
    """
        mise à niveau des élèves
    """
    usr = Eleve()
    usr.ldap_admin.connect()
    filtre = "(&%s(!(objectClass=ENTEleve)))" % ELEVE_FILTER
    eleves = usr.ldap_admin._search(filtre, ['uid'])
    for eleve in eleves:
        login = eleve[1]['uid'][0]
        try:
            usr._Upgrade(login)
        except Exception, message:
            print "Mise à niveau de l'élève %s impossible :" % login
            print str(message) + '\n'
    usr.ldap_admin.close()

def upgrade_enseignants():
    """
        mise à niveau des enseignants
    """
    usr = Enseignant()
    usr.ldap_admin.connect()
    filtre = "(&%s(!(objectClass=ENTAuxEnseignant)))" % PROF_FILTER
    profs = usr.ldap_admin._search(filtre, ['uid'])
    for prof in profs:
        login = prof[1]['uid'][0]
        try:
            usr._Upgrade(login)
        except Exception, message:
            print "Mise à niveau de l'enseignant %s impossible :" % login
            print str(message) + '\n'
    usr.ldap_admin.close()

def upgrade_mailaccounts():
    """
        migration des comptes eolemail
        vers Autres
    """
    usr = Autre()
    usr.ldap_admin.connect()
    for maildn, mailaccount in usr.ldap_admin._search("objectClass=eolemail"):
        login = mailaccount['uid'][0]
        print "Mise à niveau du compte Invité %s" % login
        args = {'login':login}
        args['password'] = 'xxx'
        args['nom'] = mailaccount['cn'][0]
        args['prenom'] = mailaccount['givenName'][0]
        args['mail'] = 'local'
        old_password = mailaccount['userPassword']
        old_mail = mailaccount['mail']
        usr.ldap_admin._delete(maildn)
        sleep(0.5)
        usr._add(**args)
        usr._set_attr(login, 'userPassword', old_password)
        usr._set_attr(login, 'mail', old_mail)
    usr.ldap_admin.close()

